@@ -71,47 +71,108 @@ DBIx::Class::Schema::Versioned - DBIx::Class::Schema plugin for Schema upgrades
 =head1 SYNOPSIS
 
   package Library::Schema;
-  use base qw/DBIx::Class::Schema/;   
+  use base qw/DBIx::Class::Schema/;
+
+  our $VERSION = 0.001;
+
   # load Library::Schema::CD, Library::Schema::Book, Library::Schema::DVD
   __PACKAGE__->load_classes(qw/CD Book DVD/);
 
-  __PACKAGE__->load_components(qw/+DBIx::Class::Schema::Versioned/);
+  __PACKAGE__->load_components(qw/Schema::Versioned/);
   __PACKAGE__->upgrade_directory('/path/to/upgrades/');
-  __PACKAGE__->backup_directory('/path/to/backups/');
 
 
 =head1 DESCRIPTION
 
-This module is a component designed to extend L<DBIx::Class::Schema>
-classes, to enable them to upgrade to newer schema layouts. To use this
-module, you need to have called C<create_ddl_dir> on your Schema to
-create your upgrade files to include with your delivery.
+This module provides methods to apply DDL changes to your database using SQL
+diff files. Normally these diff files would be created using
+L<DBIx::Class::Schema/create_ddl_dir>.
 
 A table called I<dbix_class_schema_versions> is created and maintained by the
-module. This contains two fields, 'Version' and 'Installed', which
-contain each VERSION of your Schema, and the date+time it was installed.
+module. This is used to determine which version your database is currently at.
+Similarly the $VERSION in your DBIC schema class is used to determine the
+current DBIC schema version.
 
-The actual upgrade is called manually by calling C<upgrade> on your
-schema object. Code is run at connect time to determine whether an
-upgrade is needed, if so, a warning "Versions out of sync" is
-produced.
-
-So you'll probably want to write a script which generates your DDLs and diffs
-and another which executes the upgrade.
+The upgrade is initiated manually by calling C<upgrade> on your schema object,
+this will attempt to upgrade the database from its current version to the current
+schema version using a diff from your I<upgrade_directory>. If a suitable diff is
+not found then no upgrade is possible.
 
 NB: At the moment, only SQLite and MySQL are supported. This is due to
 spotty behaviour in the SQL::Translator producers, please help us by
-them.
+enhancing them. Ask on the mailing list or IRC channel for details (community details
+in L<DBIx::Class>).
 
-=head1 METHODS
+=head1 GETTING STARTED
 
-=head2 upgrade_directory
+Firstly you need to setup your schema class as per the L</SYNOPSIS>, make sure
+you have specified an upgrade_directory and an initial $VERSION.
 
-Use this to set the directory your upgrade files are stored in.
+Then you'll need two scripts, one to create DDL files and diffs and another to perform
+upgrades. Your creation script might look like a bit like this:
 
-=head2 backup_directory
+  use strict;
+  use Pod::Usage;
+  use Getopt::Long;
+  use MyApp::Schema;
 
-Use this to set the directory you want your backups stored in.
+  my ( $preversion, $help ); 
+  GetOptions(
+    'p|preversion:s'  => \$preversion,
+  ) or die pod2usage;
+
+  my $schema = MyApp::Schema->connect(
+    $dsn,
+    $user,
+    $password,
+  );
+  my $sql_dir = './sql';
+  my $version = $schema->schema_version();
+  $schema->create_ddl_dir( 'MySQL', $version, $sql_dir, $preversion );
+
+Then your upgrade script might look like so:
+
+  use strict;
+  use MyApp::Schema;
+
+  my $schema = MyApp::Schema->connect(
+    $dsn,
+    $user,
+    $password,
+  );
+
+  if (!$schema->get_db_version()) {
+    # schema is unversioned
+    $schema->deploy();
+  } else {
+    $schema->upgrade();
+  }
+
+The script above assumes that if the database is unversioned then it is empty
+and we can safely deploy the DDL to it. However things are not always so simple.
+
+if you want to initialise a pre-existing database where the DDL is not the same
+as the DDL for your current schema version then you will need a diff which 
+converts the database's DDL to the current DDL. The best way to do this is
+to get a dump of the database schema (without data) and save that in your
+SQL directory as version 0.000 (the filename must be as with
+L<DBIx::Class::Schema/ddl_filename>) then create a diff using your create DDL 
+script given above from version 0.000 to the current version. Then hand check
+and if necessary edit the resulting diff to ensure that it will apply. Once you have 
+done all that you can do this:
+
+  if (!$schema->get_db_version()) {
+    # schema is unversioned
+    $schema->install("0.000");
+  }
+
+  # this will now apply the 0.000 to current version diff
+  $schema->upgrade();
+
+In the case of an unversioned database the above code will create the
+dbix_class_schema_versions table and write version 0.000 to it, then 
+upgrade will then apply the diff we talked about creating in the previous paragraph
+and then you're good to go.
 
 =cut
 
@@ -129,30 +190,177 @@ __PACKAGE__->mk_classdata('backup_directory');
 __PACKAGE__->mk_classdata('do_backup');
 __PACKAGE__->mk_classdata('do_diff_on_init');
 
-=head2 schema_version
 
-Returns the current schema class' $VERSION; does -not- use $schema->VERSION
-since that varies in results depending on if version.pm is installed, and if
-so the perl or XS versions. If you want this to change, bug the version.pm
-author to make vpp and vxs behave the same.
+=head1 METHODS
+
+=head2 upgrade_directory
+
+Use this to set the directory your upgrade files are stored in.
+
+=head2 backup_directory
+
+Use this to set the directory you want your backups stored in (note that backups
+are disabled by default).
 
 =cut
 
-sub schema_version {
-  my ($self) = @_;
-  my $class = ref($self)||$self;
-  my $version;
-  {
-    no strict 'refs';
-    $version = ${"${class}::VERSION"};
+=head2 install
+
+=over 4
+
+=item Arguments: $db_version
+
+=back
+
+Call this to initialise a previously unversioned database. The table 'dbix_class_schema_versions' will be created which will be used to store the database version.
+
+Takes one argument which should be the version that the database is currently at. Defaults to the return value of L</schema_version>.
+
+See L</getting_started> for more details.
+
+=cut
+
+sub install
+{
+  my ($self, $new_version) = @_;
+
+  # must be called on a fresh database
+  if ($self->get_db_version()) {
+    warn 'Install not possible as versions table already exists in database';
   }
-  return $version;
+
+  # default to current version if none passed
+  $new_version ||= $self->schema_version();
+
+  if ($new_version) {
+    # create versions table and version row
+    $self->{vschema}->deploy;
+    $self->_set_db_version;
+  }
+}
+
+=head2 deploy
+
+Same as L<DBIx::Class::Schema/deploy> but also calls C<install>.
+
+=cut
+
+sub deploy {
+  my $self = shift;
+  $self->next::method(@_);
+  $self->install();
+}
+
+=head2 upgrade
+
+Call this to attempt to upgrade your database from the version it is at to the version
+this DBIC schema is at. If they are the same it does nothing.
+
+It requires an SQL diff file to exist in you I<upgrade_directory>, normally you will
+have created this using L<DBIx::Class::Schema/create_ddl_dir>.
+
+If successful the dbix_class_schema_versions table is updated with the current
+DBIC schema version.
+
+=cut
+
+sub upgrade
+{
+  my ($self) = @_;
+  my $db_version = $self->get_db_version();
+
+  # db unversioned
+  unless ($db_version) {
+    warn 'Upgrade not possible as database is unversioned. Please call install first.';
+    return;
+  }
+
+  # db and schema at same version. do nothing
+  if ($db_version eq $self->schema_version) {
+    print "Upgrade not necessary\n";
+    return;
+  }
+
+  # strangely the first time this is called can
+  # differ to subsequent times. so we call it 
+  # here to be sure.
+  # XXX - just fix it
+  $self->storage->sqlt_type;
+  
+  my $upgrade_file = $self->ddl_filename(
+                                         $self->storage->sqlt_type,
+                                         $self->schema_version,
+                                         $self->upgrade_directory,
+                                         $db_version,
+                                        );
+
+  unless (-f $upgrade_file) {
+    warn "Upgrade not possible, no upgrade file found ($upgrade_file), please create one\n";
+    return;
+  }
+
+  # backup if necessary then apply upgrade
+  $self->_filedata($self->_read_sql_file($upgrade_file));
+  $self->backup() if($self->do_backup);
+  $self->txn_do(sub { $self->do_upgrade() });
+
+  # set row in dbix_class_schema_versions table
+  $self->_set_db_version;
+}
+
+=head2 do_upgrade
+
+This is an overwritable method used to run your upgrade. The freeform method
+allows you to run your upgrade any way you please, you can call C<run_upgrade>
+any number of times to run the actual SQL commands, and in between you can
+sandwich your data upgrading. For example, first run all the B<CREATE>
+commands, then migrate your data from old to new tables/formats, then 
+issue the DROP commands when you are finished. Will run the whole file as it is by default.
+
+=cut
+
+sub do_upgrade
+{
+  my ($self) = @_;
+
+  # just run all the commands (including inserts) in order                                                        
+  $self->run_upgrade(qr/.*?/);
+}
+
+=head2 run_upgrade
+
+ $self->run_upgrade(qr/create/i);
+
+Runs a set of SQL statements matching a passed in regular expression. The
+idea is that this method can be called any number of times from your
+C<do_upgrade> method, running whichever commands you specify via the
+regex in the parameter. Probably won't work unless called from the overridable
+do_upgrade method.
+
+=cut
+
+sub run_upgrade
+{
+    my ($self, $stm) = @_;
+
+    return unless ($self->_filedata);
+    my @statements = grep { $_ =~ $stm } @{$self->_filedata};
+    $self->_filedata([ grep { $_ !~ /$stm/i } @{$self->_filedata} ]);
+
+    for (@statements)
+    {      
+        $self->storage->debugobj->query_start($_) if $self->storage->debug;
+        $self->storage->dbh->do($_) or warn "SQL was:\n $_";
+        $self->storage->debugobj->query_end($_) if $self->storage->debug;
+    }
+
+    return 1;
 }
 
 =head2 get_db_version
 
 Returns the version that your database is currently at. This is determined by the values in the
-dbix_class_schema_versions table that $self->upgrade writes to.
+dbix_class_schema_versions table that C<upgrade> and C<install> write to.
 
 =cut
 
@@ -169,17 +377,11 @@ sub get_db_version
     return $version;
 }
 
-sub _source_exists
-{
-    my ($self, $rs) = @_;
+=head2 schema_version
 
-    my $c = eval {
-        $rs->search({ 1, 0 })->count;
-    };
-    return 0 if $@ || !defined $c;
+Returns the current schema class' $VERSION
 
-    return 1;
-}
+=cut
 
 =head2 backup
 
@@ -190,6 +392,8 @@ database type.
 
 This method should return the name of the backup file, if appropriate..
 
+This method is disabled by default. Set $schema->do_backup(1) to enable it.
+
 =cut
 
 sub backup
@@ -199,6 +403,70 @@ sub backup
     $self->storage->backup($self->backup_directory());
 }
 
+=head2 connection
+
+Overloaded method. This checks the DBIC schema version against the DB version and
+warns if they are not the same or if the DB is unversioned. It also provides
+compatibility between the old versions table (SchemaVersions) and the new one
+(dbix_class_schema_versions).
+
+To avoid the checks on connect, set the env var DBIC_NO_VERSION_CHECK or alternatively you can set the ignore_version attr in the forth argument like so:
+
+  my $schema = MyApp::Schema->connect(
+    $dsn,
+    $user,
+    $password,
+    { ignore_version => 1 },
+  );
+
+=cut
+
+sub connection {
+  my $self = shift;
+  $self->next::method(@_);
+  $self->_on_connect($_[3]);
+  return $self;
+}
+
+sub _on_connect
+{
+  my ($self, $args) = @_;
+
+  $args = {} unless $args;
+  $self->{vschema} = DBIx::Class::Version->connect(@{$self->storage->connect_info()});
+  my $vtable = $self->{vschema}->resultset('Table');
+
+  # check for legacy versions table and move to new if exists
+  my $vschema_compat = DBIx::Class::VersionCompat->connect(@{$self->storage->connect_info()});
+  unless ($self->_source_exists($vtable)) {
+    my $vtable_compat = $vschema_compat->resultset('TableCompat');
+    if ($self->_source_exists($vtable_compat)) {
+      $self->{vschema}->deploy;
+      map { $vtable->create({ installed => $_->Installed, version => $_->Version }) } $vtable_compat->all;
+      $self->storage->dbh->do("DROP TABLE " . $vtable_compat->result_source->from);
+    }
+  }
+
+  # useful when connecting from scripts etc
+  return if ($args->{ignore_version} || ($ENV{DBIC_NO_VERSION_CHECK} && !exists $args->{ignore_version}));
+  my $pversion = $self->get_db_version();
+
+  if($pversion eq $self->schema_version)
+    {
+#         warn "This version is already installed\n";
+        return 1;
+    }
+
+  if(!$pversion)
+    {
+        warn "Your DB is currently unversioned. Please call upgrade on your schema to sync the DB.\n";
+        return 1;
+    }
+
+  warn "Versions out of sync. This is " . $self->schema_version . 
+    ", your database contains version $pversion, please call upgrade on your Schema.\n";
+}
+
 # is this just a waste of time? if not then merge with DBI.pm
 sub _create_db_to_schema_diff {
   my $self = shift;
@@ -246,8 +514,8 @@ sub _create_db_to_schema_diff {
 
   my $filename = $self->ddl_filename(
                                          $db,
-                                         $self->upgrade_directory,
                                          $self->schema_version,
+                                         $self->upgrade_directory,
                                          'PRE',
                                     );
   my $file;
@@ -262,64 +530,6 @@ sub _create_db_to_schema_diff {
   print "WARNING: There may be differences between your DB and your DBIC schema. Please review and if necessary run the SQL in $filename to sync your DB.\n";
 }
 
-=head2 upgrade
-
-Call this to attempt to upgrade your database from the version it is at to the version
-this DBIC schema is at. 
-
-It requires an SQL diff file to exist in $schema->upgrade_directory, normally you will
-have created this using $schema->create_ddl_dir.
-
-=cut
-
-sub upgrade
-{
-  my ($self) = @_;
-  my $db_version = $self->get_db_version();
-
-  # db unversioned
-  unless ($db_version) {
-    # set version in dbix_class_schema_versions table, can't actually upgrade as we don 't know what version the DB is at
-    $self->_create_db_to_schema_diff() if ($self->do_diff_on_init);
-
-    # create versions table and version row
-    $self->{vschema}->deploy;
-    $self->_set_db_version;
-    return;
-  }
-
-  # db and schema at same version. do nothing
-  if ($db_version eq $self->schema_version) {
-    print "Upgrade not necessary\n";
-    return;
-  }
-
-  # strangely the first time this is called can
-  # differ to subsequent times. so we call it 
-  # here to be sure.
-  # XXX - just fix it
-  $self->storage->sqlt_type;
-  
-  my $upgrade_file = $self->ddl_filename(
-                                         $self->storage->sqlt_type,
-                                         $self->upgrade_directory,
-                                         $self->schema_version,
-                                         $db_version,
-                                        );
-
-  unless (-f $upgrade_file) {
-    warn "Upgrade not possible, no upgrade file found ($upgrade_file), please create one\n";
-    return;
-  }
-
-  # backup if necessary then apply upgrade
-  $self->_filedata($self->_read_sql_file($upgrade_file));
-  $self->backup() if($self->do_backup);
-  $self->txn_do(sub { $self->do_upgrade() });
-
-  # set row in dbix_class_schema_versions table
-  $self->_set_db_version;
-}
 
 sub _set_db_version {
   my $self = shift;
@@ -342,122 +552,22 @@ sub _read_sql_file {
   @data = split(/;/, join('', @data));
   close($fh);
   @data = grep { $_ && $_ !~ /^-- / } @data;
-  @data = grep { $_ !~ /^(BEGIN TRANACTION|COMMIT)/m } @data;
+  @data = grep { $_ !~ /^(BEGIN|BEGIN TRANSACTION|COMMIT)/m } @data;
   return \@data;
 }
 
-=head2 do_upgrade
-
-This is an overwritable method used to run your upgrade. The freeform method
-allows you to run your upgrade any way you please, you can call C<run_upgrade>
-any number of times to run the actual SQL commands, and in between you can
-sandwich your data upgrading. For example, first run all the B<CREATE>
-commands, then migrate your data from old to new tables/formats, then 
-issue the DROP commands when you are finished.
-
-Will run the whole file as it is by default.
-
-=cut
-
-sub do_upgrade
+sub _source_exists
 {
-    my ($self) = @_;
+    my ($self, $rs) = @_;
 
-    ## overridable sub, per default just run all the commands.
-    $self->run_upgrade(qr/create/i);
-    $self->run_upgrade(qr/alter table .*? add/i);
-    $self->run_upgrade(qr/alter table .*? (?!drop)/i);
-    $self->run_upgrade(qr/alter table .*? drop/i);
-    $self->run_upgrade(qr/drop/i);
-}
-
-=head2 run_upgrade
-
- $self->run_upgrade(qr/create/i);
-
-Runs a set of SQL statements matching a passed in regular expression. The
-idea is that this method can be called any number of times from your
-C<upgrade> method, running whichever commands you specify via the
-regex in the parameter. Probably won't work unless called from the overridable
-do_upgrade method.
-
-=cut
-
-sub run_upgrade
-{
-    my ($self, $stm) = @_;
-
-    return unless ($self->_filedata);
-    my @statements = grep { $_ =~ $stm } @{$self->_filedata};
-    $self->_filedata([ grep { $_ !~ /$stm/i } @{$self->_filedata} ]);
-
-    for (@statements)
-    {      
-        $self->storage->debugobj->query_start($_) if $self->storage->debug;
-        $self->storage->dbh->do($_) or warn "SQL was:\n $_";
-        $self->storage->debugobj->query_end($_) if $self->storage->debug;
-    }
+    my $c = eval {
+        $rs->search({ 1, 0 })->count;
+    };
+    return 0 if $@ || !defined $c;
 
     return 1;
 }
 
-=head2 connection
-
-Overloaded method. This checks the DBIC schema version against the DB version and
-warns if they are not the same or if the DB is unversioned. It also provides
-compatibility between the old versions table (SchemaVersions) and the new one
-(dbix_class_schema_versions).
-
-To avoid the checks on connect, set the env var DBIC_NO_VERSION_CHECK. This can be
-useful for scripts.
-
-=cut
-
-sub connection {
-  my $self = shift;
-  $self->next::method(@_);
-  $self->_on_connect;
-  return $self;
-}
-
-sub _on_connect
-{
-  my ($self) = @_;
-  $self->{vschema} = DBIx::Class::Version->connect(@{$self->storage->connect_info()});
-  my $vtable = $self->{vschema}->resultset('Table');
-
-  # check for legacy versions table and move to new if exists
-  my $vschema_compat = DBIx::Class::VersionCompat->connect(@{$self->storage->connect_info()});
-  unless ($self->_source_exists($vtable)) {
-    my $vtable_compat = $vschema_compat->resultset('TableCompat');
-    if ($self->_source_exists($vtable_compat)) {
-      $self->{vschema}->deploy;
-      map { $vtable->create({ installed => $_->Installed, version => $_->Version }) } $vtable_compat->all;
-      $self->storage->dbh->do("DROP TABLE " . $vtable_compat->result_source->from);
-    }
-  }
-  
-  # useful when connecting from scripts etc
-  return if ($ENV{DBIC_NO_VERSION_CHECK});
-  
-  my $pversion = $self->get_db_version();
-
-  if($pversion eq $self->schema_version)
-    {
-#         warn "This version is already installed\n";
-        return 1;
-    }
-
-  if(!$pversion)
-    {
-        warn "Your DB is currently unversioned. Please call upgrade on your schema to sync the DB.\n";
-        return 1;
-    }
-
-  warn "Versions out of sync. This is " . $self->schema_version . 
-    ", your database contains version $pversion, please call upgrade on your Schema.\n";
-}
-
 1;
 
 
